import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:random/cubit/api_cubit.dart';
import 'package:random/cubit/bottom_nav_cubit.dart';
import 'package:random/cubit/data_cubit.dart';
import 'package:random/cubit/game_cubit.dart';
import 'package:random/cubit/settings_cubit.dart';
import 'package:random/repository/api.dart';
import 'package:random/network/api.dart';
import 'package:random/ui/skeleton.dart';

void main() async {
  /// Initialize packages
  WidgetsFlutterBinding.ensureInitialized();
  await EasyLocalization.ensureInitialized();
  final Directory tmpDir = await getTemporaryDirectory();
  Hive.init(tmpDir.toString());
  HydratedBloc.storage = await HydratedStorage.build(
    storageDirectory: tmpDir,
  );

  runApp(
    EasyLocalization(
      path: 'assets/translations',
      supportedLocales: const <Locale>[
        Locale('en'),
        Locale('fr'),
      ],
      fallbackLocale: const Locale('en'),
      useFallbackTranslations: true,
      child: const MyApp(),
    ),
  );
}

class MyApp extends StatelessWidget {
  const MyApp({super.key});

  @override
  Widget build(BuildContext context) {
    return MultiBlocProvider(
      providers: [
        BlocProvider<ApiDataCubit>(
          create: (context) => ApiDataCubit(
            apiRepository: ApiRepository(
              apiService: ApiService(),
            ),
          )..fetchApiData(),
        ),
        BlocProvider<BottomNavCubit>(create: (context) => BottomNavCubit()),
        BlocProvider<DataCubit>(create: (context) => DataCubit()),
        BlocProvider<GameCubit>(create: (context) => GameCubit()),
        BlocProvider<SettingsCubit>(create: (context) => SettingsCubit()),
        BlocProvider<ApplicationThemeModeCubit>(
            create: (context) => ApplicationThemeModeCubit()),
      ],
      child: BlocBuilder<ApplicationThemeModeCubit, ApplicationThemeModeState>(
          builder: (BuildContext context, ApplicationThemeModeState state) {
        return MaterialApp(
          title: 'Random application',
          theme: lightTheme,
          darkTheme: darkTheme,
          themeMode: state.themeMode,
          home: const SkeletonScreen(),
          localizationsDelegates: context.localizationDelegates,
          supportedLocales: context.supportedLocales,
          locale: context.locale,
          debugShowCheckedModeBanner: false,
        );
      }),
    );
  }
}
